/* Copyright (C) 2006, 2007, 2008, 2009 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package compbase;

/**
 * ExArrayInit: expression containing constant array initialization
 * 
 * @author S. Frenz
 * @version 091005 moved to compbase-package
 *  version 090724 adopted changed Expr
 *  version 090207 added copyright notice
 *  version 080616 added filling of dependsOn, removed getCompInitConstObjectVrbl
 *  version 080614 adopted move of arrays from Clss to JUnit
 *  version 070918 added support for multidimensional arrays
 *  version 070909 optimized signature of Expr.resolve, adopted changes in ExAccVrbl
 *  version 070727 replaced exSubResolve by resolve as there is nothing done anymore
 *  version 070628 adopted changes for "Expr extends TypeRef"
 *  version 070511 removed invalid location assignment
 *  version 070114 reduced access level where possible
 *  version 070111 adapted change in printPos and compErr
 *  version 070106 removed genOutputAddr
 *  version 061203 optimized calls to printPos and compErr
 *  version 061202 optimized static modes
 *  version 061130 initial version
 */

public class ExArrayInit extends ExConstInitObj {
  public int len;
  public FilledParam par;
  public ExArrayInit nextArrayInit; //used to chain arrays in a class
  
  public ExArrayInit(int fid, int il, int ic) {
    super(fid, il, ic);
  }

  public boolean resolve(Unit unitContext, Mthd mthdContext, boolean checkRead, boolean insideLoop,
      TypeRef preferredType, Context ctx) {
    FilledParam pa;
    TypeRef element=null;
    Expr ex;
    TypeRef preferredElemType=null;
    
    //enter in array-list of unit (has to be done before param-resolve to ensure correct order in list)
    target.owner=unitContext;
    if (mthdContext!=unitContext.initStat && mthdContext!=unitContext.initDyna) dependsOn=mthdContext;
    nextArrayInit=unitContext.arrays;
    unitContext.arrays=this;
    //check parameter and enter type
    if (preferredType!=null) {
      preferredElemType=new TypeRef(fileID, line, col);
      preferredElemType.getTypeOf(preferredType);
      if (--preferredElemType.arrDim<0) preferredElemType=null;
    }
    pa=par;
    len=0;
    while (pa!=null) {
      len++;
      ex=pa.expr; //get expression
      pa=pa.nextParam; //switch to next expression to enable "continue" below
      if (!ex.resolve(unitContext, mthdContext, true, insideLoop, preferredElemType, ctx)) return false;
      if (!(ex instanceof ExConstInitObj)) {
        switch (ex.calcConstantType(ctx)) {
          case 0:
            if (ex.baseType==StdTypes.T_NULL) continue; //null-type valid, do not copy type
            ex.printPos(ctx, "array initialization needs constant value, array or String");
            return false;
          case StdTypes.T_INT: case StdTypes.T_LONG:
            break;
          default:
            ex.compErr(ctx, "unknown or invalid constant type in ExArrayInit.exSubResolve");
            return false;
        }
      }
      if (element==null) element=ex;
      else if (element.compareType(ex, false, ctx)!=TypeRef.C_EQ) {
        printPos(ctx, "array initialization needs identical typed values");
        return false;
      }
    }
    if (len==0 || element==null) {
      printPos(ctx, "empty-/null-initialization not supported");
      return false;
    }
    getTypeOf(element);
    arrDim++;
    target.type=this;
    constObject=true;
    return true;
  }

  public void genOutputVal(int reg, Context ctx) {
    compErr(ctx, "ExArrayInit.genOutputVal not supported");
  }
}
