/* Copyright (C) 2005, 2006, 2007, 2008, 2009 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package osio;

/**
 * TextReader: interface to read text
 * 
 * @author S. Frenz
 * @version 090303 adopted changed osio package structure
 *  version 090207 added copyright notice
 *  version 080202 removed eof as it is not required anymore
 *  version 070331 got system independent code
 *  version 060712 streamlined version
 *  version 060607 initial version
 */

public abstract class TextReader {
	public char nextChar, lookAhead;
	public int line, col;
	
  protected boolean errorOccurred;
  protected byte[] data;
  protected int pos, len;

  protected abstract boolean systemReadData(String fname);

  public boolean initData(String fname) {
    nextChar='\0';
    lookAhead='\0';
    line=1;
    col=0;
    pos=0;
    if (errorOccurred=!systemReadData(fname)) return false;
    readChar();
    return true;
  }
  
  public void readChar() {
    int b;
    
    if (errorOccurred) return;
    if (pos>=len) {
      if (nextChar=='\0' && lookAhead=='\0') errorOccurred=true;
      nextChar=lookAhead;
      lookAhead='\0';
    }
    else {
      b=(int)data[pos++]&0xFF;
      if ((nextChar=='\0') || (lookAhead=='\0')) {
        lookAhead=(char)b;
        b=(int)data[pos++]&0xFF;
      }
      nextChar=lookAhead;
      lookAhead=(char)b;
      col++;
    }
    if ((nextChar=='\r' && lookAhead!='\n') || nextChar=='\n') {
      line++;
      col=0;
    }
  }
}
