/* Copyright (C) 2009 Patrick Schmidt, Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package relations;

import compbase.Mthd;
import compbase.Unit;

/**
 * RelationManager: keeps track of relations during compilation
 * 
 * @author P. Schmidt, S. Frenz
 * @version 091009 initial version
 */

public class RelationManager {
  
  public static final String INHERITANCE="<cInheritance>";
  public static final String INTERFACE="<interface>";
  public static final String ROOT_OBJECT="Object";
  
  public RelationElement index=null;
  
  public void removeRelation(Unit u) {
    RelationElement loop, inner;
    //take care if it's the first unit in the list
    while (index!=null && index.unit==u) index=index.next;
    loop=index;
    while (loop!=null) {
      //otherwise check if somewhere in the list
      while (loop.next!=null && loop.next.unit==u) loop.next=loop.next.next;
      //check referencedBy-list
      while (loop.referencedBy!=null && loop.referencedBy.unit==u)
        loop.referencedBy=loop.referencedBy.next;
      inner=loop.referencedBy;
      while (inner!=null) {
        if (inner.next!=null && inner.next.unit==u) inner.next=inner.next.next;
        else inner=inner.next;
      }
      loop=loop.next;
    }
  }
  
  public boolean isStructuralDependance(String s) {
    if (s==null) return false;
    return s.equals(INHERITANCE) || s.equals(INTERFACE);
  }
  
  public void addInterface(Unit ifc, Unit implementing) {
    insert(ifc, INTERFACE, implementing, null, false);
  }
  
  public void addInheritance(Unit from, Unit to) {
    if (from.name.equals(ROOT_OBJECT)) return;
    insert(from, INHERITANCE, to, null, false);
  }
  
  public void updateRelation(Unit toReplace, Unit with) {
    RelationElement inner, outer=index;
    while (outer!=null) {
      if (outer.unit==toReplace) outer.unit=with;
      inner=outer.referencedBy;
      while (inner!=null) {
        if (inner.unit==toReplace) inner.unit=with;
        inner=inner.next;
      }
      outer=outer.next;
    }
  }
  
  public void addMethodRelation(Unit usedUnit, String usedMthd, Unit usingUnit, 
      Mthd usingMthd) {
    insert(usedUnit, usedMthd, usingUnit, usingMthd, false);
  }
  
  public void addVariableRelation(Unit usedUnit, String usedVar, Unit usingUnit,
      Mthd usingMthd) {
    insert(usedUnit, usedVar, usingUnit, usingMthd, true);
  }
  
  private void insert(Unit usedUnit, String usedItem, Unit usingUnit, Mthd usingMthd,
      boolean isVar) {
    RelationElement outer=index, inner;
    if (usedUnit==null || usedItem==null || usingUnit==null /*|| usingMthd==null*/) return;
    if (usingUnit.name==null || usingUnit.name==null/* || usingMthd.name==null*/) return;
    while (outer!=null) {
      if ((outer.isVar==isVar) && outer.unit==usedUnit && outer.mthd.equals(usedItem)) {
        inner=outer.referencedBy;
        while (inner!=null) {
          if (inner.unit==usingUnit && (inner.mthd==null || inner.mthd.equals(usingMthd.name))) return;
          inner=inner.next;
        }
        inner=new RelationElement();
        inner.next=outer.referencedBy;
        inner.unit=usingUnit;//.name;
        inner.mthd=usingMthd==null ? "<null>" : usingMthd.name;
        inner.isVar=isVar;
        outer.referencedBy=inner;
        return;
      }
      outer=outer.next;
    }
    outer=new RelationElement();
    outer.unit=usedUnit;//.name;
    outer.mthd=usedItem;
    outer.isVar=isVar;
    outer.next=index;
    outer.referencedBy=new RelationElement();
    outer.referencedBy.isVar=isVar;
    outer.referencedBy.unit=usingUnit;//.name;
    outer.referencedBy.mthd=usingMthd==null ? "<null>" : usingMthd.name;
    index=outer;    
  }  
}
